<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://www.smartbill.ro
 * @since             1.0.0
 * @copyright         Intelligent IT SRL 2018
 * @package           smartbill-facturare-si-gestiune
 *
 * @wordpress-plugin
 * Plugin Name:       SmartBill Facturare si Gestiune
 * Plugin URI:        https://www.smartbill.ro/resurse/integrari
 * Description:       Acest modul permite emiterea facturilor in SmartBill Cloud pentru WooCommerce 3.0 - 7.1.0, WordPress 4.7.0 - 6.1.1
 * Version:           3.0.1
 * Author:            smarbill.ro
 * Author URI:        https://www.smartbill.ro
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       smartbill-woocommerce
 * Domain Path:       /languages
 * Copyright:         Intelligent IT SRL 2018
 * Requires at least: 4.7.0
 * Tested up to:      6.1.1
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die();
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'SMARTBILL_PLUGIN_VERSION', '3.0.1' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-smartbill-woocommerce-activator.php
 */
function activate_smartbill_woocommerce() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-smartbill-woocommerce-activator.php';
	Smartbill_Woocommerce_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-smartbill-woocommerce-deactivator.php
 */
function deactivate_smartbill_woocommerce() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-smartbill-woocommerce-deactivator.php';
	Smartbill_Woocommerce_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_smartbill_woocommerce' );
register_deactivation_hook( __FILE__, 'deactivate_smartbill_woocommerce' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-smartbill-woocommerce.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_smartbill_woocommerce() {

	$plugin = new Smartbill_Woocommerce();
	$plugin->run();

}
run_smartbill_woocommerce();

/**
 * Show error message */
function show_smartbill_version_err() {
	$allowed_html = array(
		'a'      => array(
			'href'   => array(),
			'target' => array(),
		),
		'br'     => array(),
		'em'     => array(),
		'strong' => array(),
	);
	/* translators: Variable is WordPress version-version*/
	$part_1 = sprintf( __( 'Modulul SmartBill necesita %s', 'smartbill-woocommerce' ), '<strong>WordPress 4.7 - 6.1.1</strong>' );
	/* translators: Variable is SmartBill url*/
	$part_2 = sprintf( __( 'Acceseaza %s', 'smartbill-woocommerce' ), '<strong><a target="_blank" href="https://api.smartbill.ro/plugins.html#woocommerce">https://api.smartbill.ro/plugins.html#woocommerce</a></strong>' );
	/* translators: Variable is SmartBill email address*/
	$part_3 = sprintf( __( 'Daca modulul nu este compatibil cu ultima versiune de WordPress sau WooCommerce, contacteaza-ne la %s', 'smartbill-woocommerce' ), '<a href="mailto:plugins@smartbill.ro">plugins@smartbill.ro</a>' );

	wp_die( wp_kses( $part_1, $allowed_html ) . '<br/><br/>' . wp_kses( $part_2, $allowed_html ) . '<br/><br/>' . wp_kses( $part_3, $allowed_html ) . '<br/><br/><br/> <a href="plugins.php" >' . wp_kses( __( 'Inapoi', 'smartbill-woocommerce' ), $allowed_html ) . '</a>' );
}

/**
 * Check WordPress and WooCommerce versions
 *
 * @return boolean
 */
function check_smartbill_compatibility() {
	$min_wp_ver  = '4.7.0';
	$max_wp_ver  = '6.1.1';
	$min_woo_ver = '3.0';
	$max_woo_ver = '7.1.0';

	// check woocommerce version to be higher than 7.1.0.
	if ( version_compare( $GLOBALS['wp_version'], $min_wp_ver, '<' ) || version_compare( $GLOBALS['wp_version'], $max_wp_ver, '>' ) ) {
		return false;
	}
	if ( class_exists( 'WooCommerce' ) ) {
		global $woocommerce;
		if ( version_compare( $woocommerce->version, $min_woo_ver, '<=' ) ) {
			return false;
		}
	} else {
		return false;
	}

	// Add sanity checks for other version requirements here.
	return true;
}


